package com.yonge.cooleshow.admin.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.yonge.cooleshow.admin.dao.OmsOrderReturnApplyDao;
import com.yonge.cooleshow.admin.dto.OmsOrderReturnApplyResult;
import com.yonge.cooleshow.admin.dto.OmsReturnApplyQueryParam;
import com.yonge.cooleshow.admin.dto.OmsUpdateStatusParam;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.cooleshow.mbg.mapper.OmsOrderReturnApplyMapper;
import com.yonge.cooleshow.mbg.mapper.UserOrderPaymentMapper;
import com.yonge.cooleshow.mbg.mapper.UserOrderRefundMapper;
import com.yonge.cooleshow.mbg.model.OmsOrderReturnApply;
import com.yonge.cooleshow.mbg.model.OmsOrderReturnApplyExample;
import com.yonge.cooleshow.admin.service.OmsOrderReturnApplyService;
import com.yonge.cooleshow.mbg.model.UserOrderPayment;
import com.yonge.cooleshow.mbg.model.UserOrderRefund;
import com.yonge.toolset.base.result.BaseResult;
import com.yonge.toolset.payment.base.enums.OpenEnum;
import com.yonge.toolset.payment.base.enums.PayChannelEnum;
import com.yonge.toolset.payment.base.enums.TradeStatusEnum;
import com.yonge.toolset.payment.base.model.RefundBill;
import com.yonge.toolset.payment.core.service.PaymentClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;

/**
 * 订单退货管理Service实现类
 * Created by macro on 2018/10/18.
 */
@Service
public class OmsOrderReturnApplyServiceImpl implements OmsOrderReturnApplyService {

    private static final Logger log = LoggerFactory.getLogger(OmsOrderReturnApplyService.class);

    @Autowired
    private OmsOrderReturnApplyDao returnApplyDao;
    @Autowired
    private OmsOrderReturnApplyMapper returnApplyMapper;

    @Autowired
    private PaymentClient paymentClient;

    @Autowired
    private UserOrderPaymentMapper userOrderPaymentMapper;

    @Autowired
    private UserOrderRefundMapper userOrderRefundMapper;

    @Override
    public List<OmsOrderReturnApply> list(OmsReturnApplyQueryParam queryParam, Integer pageSize, Integer pageNum) {
        PageHelper.startPage(pageNum,pageSize);
        return returnApplyDao.getList(queryParam);
    }

    @Override
    public int delete(List<Long> ids) {
        OmsOrderReturnApplyExample example = new OmsOrderReturnApplyExample();
        example.createCriteria().andIdIn(ids).andStatusEqualTo(3);
        return returnApplyMapper.deleteByExample(example);
    }

    @Override
    public int updateStatus(Long id, OmsUpdateStatusParam statusParam) {
        Integer status = statusParam.getStatus();
        OmsOrderReturnApply omsOrderReturnApply = returnApplyMapper.selectByPrimaryKey(id);
        if (omsOrderReturnApply.getStatus().equals(statusParam.getStatus())) {
            return 1;
        }
        OmsOrderReturnApply returnApply = new OmsOrderReturnApply();
        if(status.equals(1)){
            //确认退货
            returnApply.setId(id);
            returnApply.setStatus(1);
            returnApply.setReturnAmount(statusParam.getReturnAmount());
            returnApply.setCompanyAddressId(statusParam.getCompanyAddressId());
            returnApply.setHandleTime(new Date());
            returnApply.setHandleMan(statusParam.getHandleMan());
            returnApply.setHandleNote(statusParam.getHandleNote());
        }else if(status.equals(2)){
            //完成退货
            returnApply.setId(id);
            returnApply.setStatus(2);
            returnApply.setReceiveTime(new Date());
            returnApply.setReceiveMan(statusParam.getReceiveMan());
            returnApply.setReceiveNote(statusParam.getReceiveNote());
            refundAmount(returnApply);
        }else if(status.equals(3)){
            //拒绝退货
            returnApply.setId(id);
            returnApply.setStatus(3);
            returnApply.setHandleTime(new Date());
            returnApply.setHandleMan(statusParam.getHandleMan());
            returnApply.setHandleNote(statusParam.getHandleNote());
        }else{
            return 0;
        }
        return returnApplyMapper.updateByPrimaryKeySelective(returnApply);
    }

    /**
     * 退款处理
     *
     * @param returnApply 退款信息
     */
    private void refundAmount(OmsOrderReturnApply returnApply) {
        // 退款
        UserOrderPayment userOrderPayment = userOrderPaymentMapper.selectByOrderNoAndStatusSucceeded(returnApply.getOrderSn());
        RefundBill refundReq = new RefundBill(OpenEnum.valueOf(userOrderPayment.getOpenType()), PayChannelEnum.valueOf(userOrderPayment.getPayChannel()));
        refundReq.setRefundNo(returnApply.getOrderSn());
        refundReq.setRefundAmt(returnApply.getReturnAmount().setScale(2, RoundingMode.HALF_UP));
        refundReq.setReason(returnApply.getProductName());
        refundReq.setPaymentNo(userOrderPayment.getAdapayNo());
        refundReq.setTradeNo(userOrderPayment.getTransNo());
        refundReq.setOrderAmt(userOrderPayment.getPayAmt());
        BaseResult<RefundBill> refundBillBaseResult = paymentClient.refundPayment(refundReq);
        if (refundBillBaseResult.getStatus()) {
            RefundBill refundBill = refundBillBaseResult.getData();
            UserOrderRefund userOrderRefund = new UserOrderRefund();
            userOrderRefund.setOrderNo(refundBill.getRefundNo());
            userOrderRefund.setTransNo(refundBill.getId());
            userOrderRefund.setPayTransNo(refundBill.getTradeNo());
            userOrderRefund.setRefundAmt(refundBill.getRefundAmt());
            userOrderRefund.setStatus(TradeStatusEnum.pending.getCode());
            userOrderRefundMapper.insert(userOrderRefund);
        } else {
            throw new BizException(refundBillBaseResult.getMsg());
        }
    }

    @Override
    public OmsOrderReturnApplyResult getItem(Long id) {
        return returnApplyDao.getDetail(id);
    }
}
