package com.yonge.cooleshow.biz.dal.sdk;

import com.alibaba.fastjson.JSONObject;
import com.yonge.cooleshow.biz.dal.props.WithdrawalProperties;
import com.yonge.toolset.thirdparty.lingxinpay.Md5EncryptUtils;
import com.yonge.toolset.thirdparty.lingxinpay.RSA;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.http.HttpUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author: cy
 * @date: 2022/5/9 11:24
 */
@Service
public class WithdrawSdk {
    private static final Logger logger = LoggerFactory.getLogger(WithdrawSdk.class);

    @Autowired
    private WithdrawalProperties withdrawalProperties;

    /**
     * 单笔提现
     *
     * @param outerOrderNo  商户唯一订单号
     * @param name          收款方姓名（银行预留姓名等）
     * @param mobile        收款方电话
     * @param certificateNo 收款方身份证号
     * @param predictAmount 应发金额（单位为：分，范围： 1~10000000000）
     * @param payAccount    收款方账号（银行卡号/支付宝账号 /open_id）以实际业务为准
     * @param remark        备注（扩展字段） 只针对测试环境：只要参数合规都会返回交易成功，如果想要模拟交易失败，该字段值中含有“失败”二字即会失败。
     * @return
     */
    public String withdraw(String outerOrderNo, String name, String mobile, String certificateNo, Integer predictAmount,
                           String payAccount, String remark) {
        Map<String, Object> map = new HashMap<>();
        map.put("outMemberNo", withdrawalProperties.getMemberNo());
        map.put("outerOrderNo", outerOrderNo);
        map.put("name", name);
        map.put("certificateNo", certificateNo);
        map.put("predictAmount", predictAmount);
        String signs = Md5EncryptUtils.sign(map, withdrawalProperties.getMd5Key());

        map.put("charset", "UTF-8");
        map.put("mobile", mobile);
        map.put("version", "1.1");
        map.put("service", "bpotop.zx.pay.order");
        map.put("Md5Key", signs);
        map.put("notifyUrl", withdrawalProperties.getNotifyUrl());
        //卡类型 DC 借记卡
        map.put("cardType", "DC");
        //输入发放类型（0：工资，1：奖金，2：绩效，3：劳务，4：个人经营所得，5：其他）
        map.put("salaryType", 4);
        map.put("projectName", name + DateUtil.format(new Date(), DateUtil.DEFAULT_PATTERN) + "提现");
        //支付类型（1：银行卡，2：支付宝，4:微信） 以实际业务为准
        map.put("payType", 1);
        //卡属性:(C:对私)
        map.put("cardAttribute", "C");
        map.put("payAccount", payAccount);
        if (StringUtils.isNotBlank(remark)) {
            map.put("remark", remark);
        }
        String jsonStr = JSONObject.toJSONString(map);

        //签名
        JSONObject mapParam = new JSONObject();
        try {
            //使用公钥加密
            String encryptStr = RSA.encryptPub(jsonStr, withdrawalProperties.getPublicKey());
            mapParam.put("outMemberNo", withdrawalProperties.getMemberNo());
            mapParam.put("signType", "RSA");
            mapParam.put("sign", encryptStr);
            logger.info("单笔请求请求参数：{}", JSONObject.toJSONString(mapParam));
        } catch (Exception e) {
            logger.info("加密失败：{}", e);
        }

        //发送
        try {
            String resultJsonStr = HttpUtil.postForHttp(withdrawalProperties.getApiUrl() + "/bpotop_trade/single", JSONObject.toJSONString(mapParam), null);
            logger.info("单笔请求返回参数：{}", resultJsonStr);
            return resultJsonStr;
        } catch (IOException e) {
            logger.info("发送失败：{}", e);
        }
        return null;
    }

    /**
     * 查询接口
     *
     * @param outerOrderNo 商户唯一订单号
     * @throws Exception
     */
    public void query(String outerOrderNo) throws Exception {
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("outMemberNo", withdrawalProperties.getMemberNo());
        requestMap.put("outerOrderNo", outerOrderNo);
        requestMap.put("service", "bpotop.zx.pay.order");
        requestMap.put("version", "1.0");
        requestMap.put("signType", "RSA");
        requestMap.put("charset", "UTF-8");
        String jsonStr = JSONObject.toJSONString(requestMap);
        String encryptStr = RSA.encryptPub(jsonStr, withdrawalProperties.getPublicKey());

        Map<String, Object> requestMap2 = new HashMap<>();
        requestMap2.put("outMemberNo", withdrawalProperties.getMemberNo());
        requestMap2.put("sign", encryptStr);
        logger.info("单笔查询请求参数：{}", JSONObject.toJSONString(requestMap2));
        String resultJsonStr = HttpUtil.postForHttp(withdrawalProperties.getApiUrl() + "/bpotop_trade/order_query", JSONObject.toJSONString(requestMap2), null);
        logger.info("单笔查询响应参数：{}", resultJsonStr);
    }

}
