package com.yonge.cooleshow.teacher.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dto.req.WithdrawalReq;
import com.yonge.cooleshow.biz.dal.dto.search.TeacherWithdrawalSearch;
import com.yonge.cooleshow.biz.dal.entity.UserWithdrawalCallback;
import com.yonge.cooleshow.biz.dal.props.WithdrawalProperties;
import com.yonge.cooleshow.biz.dal.service.UserWithdrawalService;
import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.biz.dal.vo.UserWithdrawalVo;
import com.yonge.cooleshow.biz.dal.vo.res.WithdrawalInfoRes;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.thirdparty.lingxinpay.RSA;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.json.JsonUtil;
import com.yonge.toolset.utils.string.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;
import java.util.Map;

@RestController
@RequestMapping("/userWithdrawal")
@Api(value = "用户账户提现表", tags = "用户账户提现表")
public class UserWithdrawalController extends BaseController {
    private final static Logger log = LoggerFactory.getLogger(UserWithdrawalController.class);

    @Autowired
    private UserWithdrawalService userWithdrawalService;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private WithdrawalProperties withdrawalProperties;

    @PostMapping("/getWithdrawalInfo")
    @ApiOperation(value = "查询提现页面信息")
    public HttpResponseResult<WithdrawalInfoRes> getWithdrawalInfo() {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return userWithdrawalService.getWithdrawalInfo(user.getId());
    }

    @PostMapping("/withdrawal")
    @ApiOperation(value = "用户提现")
    public HttpResponseResult<Boolean> withdrawal(@Valid @RequestBody WithdrawalReq withdrawalReq) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return userWithdrawalService.withdrawal(user, withdrawalReq);
    }

    @PostMapping("/withdrawalPage")
    @ApiOperation(value = "提现记录")
    @ApiImplicitParams({
            @ApiImplicitParam(
                    name = "status",
                    value = "状态 DOING、审核中 PASS、通过 UNPASS、不通过",
                    paramType = "query", dataType = "String"
            ),
            @ApiImplicitParam(
                    name = "searchDate",
                    value = "查询时间（yyyy-mm）",
                    paramType = "query", dataType = "String"
            )
    })
    public HttpResponseResult<PageInfo<UserWithdrawalVo>> withdrawalPage(@ApiIgnore @RequestBody TeacherWithdrawalSearch query) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        query.setUserId(user.getId());

        if (StringUtil.isEmpty(query.getSearchDate())) {
            query.setSearchDate(DateUtil.format(new Date(), "yyyy-MM"));
        }
        String[] classDateSp = query.getSearchDate().split("-");
        try {
            LocalDate date = LocalDate.of(Integer.parseInt(classDateSp[0]), Integer.parseInt(classDateSp[1]), 1);

            LocalDateTime firstDay = LocalDateTime.of(date.with(TemporalAdjusters.firstDayOfMonth()), LocalTime.MIN);
            LocalDateTime lastDay = LocalDateTime.of(date.with(TemporalAdjusters.lastDayOfMonth()), LocalTime.MAX);

            query.setStartTime(firstDay);
            query.setEndTime(lastDay);
        } catch (Exception e) {
            throw new BizException("查询时间格式不正确 [" + query.getSearchDate() + "]");
        }

        IPage<UserWithdrawalVo> pages = userWithdrawalService.selectPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
    }

    /**
     * 异步回调接收
     * @param content
     * @param request
     * @return
     */
    @PostMapping("/callback")
    public String callback(@RequestBody String content, HttpServletRequest request) {
        log.info("交易回调请求地址：{} 请求参数：{}", request.getRemoteAddr(), content);
        try {
            if (StringUtils.isBlank(content)) {
                throw new Exception();
            }
            Map<String, Object> map = JSONObject.parseObject(content);
            String jsonStr = RSA.decryptPri((String) map.get("sign"), withdrawalProperties.getPrivateKey());
            log.info("jsonStr:{}", jsonStr);

            Map<String, Object> withdrawRecord = JSONObject.parseObject(jsonStr);
            UserWithdrawalCallback callback = JsonUtil.toJavaObject(withdrawRecord, UserWithdrawalCallback.class);

            userWithdrawalService.callback(callback, jsonStr);
        } catch (BizException e) {
            log.error("解密失败e：{}", e.getMessage());
            return "failed";
        } catch (Exception e) {
            log.error("解密失败e：{}", e);
            return "failed";
        }
        return "success";
    }

}