package com.yonge.cooleshow.teacher.controller;

import com.alibaba.fastjson.JSONObject;
import com.yonge.cooleshow.biz.dal.entity.UserWithdrawalCallback;
import com.yonge.cooleshow.biz.dal.service.LingXinService;
import com.yonge.cooleshow.biz.dal.service.UserWithdrawalCallbackService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.toolset.thirdparty.lingxinpay.RSA;
import com.yonge.cooleshow.biz.dal.sdk.WithdrawSdk;
import com.yonge.toolset.utils.json.JsonUtil;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.UUID;

/**
 * @Description: 提现回调
 * @Author: cy
 * @Date: 2022/5/9
 */
@RestController
@RequestMapping("/withdraw")
@Api(value = "提现回调", tags = "提现回调")
public class WithdrawController extends BaseController {
    private final static Logger log = LoggerFactory.getLogger(WithdrawController.class);

    @Value("${withdraw.privateKey}")
    private String privateKey;//商户自己生成的私钥

    @Autowired
    private UserWithdrawalCallbackService callbackService;

    /**
     * 异步回调接收
     *
     * @param content
     * @param request
     * @return
     */
    @PostMapping("/callback")
    public String test(@RequestBody String content, HttpServletRequest request) {
        System.out.println(privateKey);
        log.info("交易回调请求地址：{} 请求参数：{}", request.getRemoteAddr(), content);
        try {
            if (StringUtils.isBlank(content)) {
                throw new Exception();
            }
            Map<String, Object> map = JSONObject.parseObject(content);
            String jsonStr = RSA.decryptPri((String) map.get("sign"), privateKey);
            log.info("jsonStr:{}", jsonStr);

            Map<String, Object> withdrawRecord = JSONObject.parseObject(jsonStr);
            UserWithdrawalCallback callback = JsonUtil.toJavaObject(withdrawRecord, UserWithdrawalCallback.class);
            callbackService.insertCallback(callback);
        } catch (Exception e) {
            log.error("解密失败e：{}", e);
            return "failed";
        }
        return "success";
    }

    /**
     * 提现测试接口
     * @param remark
     * @return
     */
    @GetMapping("/test")
    public String a(String remark) {
        WithdrawSdk withdraw = new WithdrawSdk();
        //输入商户订单号
        String outerOrderNo = UUID.randomUUID().toString().substring(0, 12);
        System.out.println("商户订单号:" + outerOrderNo);
        //输入收款人手机号
        String name = "何亮";
        //输入收款人姓名
        String mobile = "17600220933";
        //输入收款人身份证号
        String certificateNo = "130423199206192818";
        //输入转账金额（单位分）
        Integer predictAmount = 1;
        //输入收款人账号
        String payAccount = "6228480018864836772";

        String requestParam = withdraw.withdraw(outerOrderNo, name, mobile, certificateNo, predictAmount,
                payAccount, remark);
        log.info("单笔请求返回参数：{}", requestParam);
        return requestParam;
    }
}