package com.yonge.toolset.payment.original.wx;

import com.github.binarywang.wxpay.bean.notify.SignatureHeader;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyV3Result;
import com.github.binarywang.wxpay.bean.notify.WxPayRefundNotifyV3Result;
import com.github.binarywang.wxpay.bean.request.*;
import com.github.binarywang.wxpay.bean.result.WxPayOrderQueryV3Result;
import com.github.binarywang.wxpay.bean.result.WxPayRefundV3Result;
import com.github.binarywang.wxpay.bean.result.enums.TradeTypeEnum;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.github.binarywang.wxpay.service.WxPayService;
import com.yonge.toolset.base.result.BaseResult;
import com.yonge.toolset.payment.base.PaymentTemplate;
import com.yonge.toolset.payment.base.enums.MethodNameEnum;
import com.yonge.toolset.payment.base.enums.OpenEnum;
import com.yonge.toolset.payment.base.enums.PayChannelEnum;
import com.yonge.toolset.payment.base.enums.TradeStatusEnum;
import com.yonge.toolset.payment.base.model.ClosePayment;
import com.yonge.toolset.payment.base.model.OpenAuth;
import com.yonge.toolset.payment.base.model.Payment;
import com.yonge.toolset.payment.base.model.RefundBill;
import com.yonge.toolset.payment.base.model.callback.ExecutePaymentCallBack;
import com.yonge.toolset.payment.base.model.callback.PaymentCallBack;
import com.yonge.toolset.payment.base.model.callback.RefundPaymentCallBack;
import com.yonge.toolset.payment.core.props.PaymentProperties;
import com.yonge.toolset.payment.core.service.SysConfigPaymentService;
import com.yonge.toolset.payment.original.wx.constant.WxpayConstant;
import com.yonge.toolset.utils.collection.MapUtil;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.obj.ObjectUtil;
import com.yonge.toolset.utils.string.StringUtil;
import com.yonge.toolset.utils.web.WebUtil;
import org.apache.commons.collections.map.HashedMap;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

@Component
public class OriginalWxAppTemplate implements PaymentTemplate {

    private final static Logger log = LoggerFactory.getLogger(OriginalWxAppTemplate.class);

    @Autowired
    private PaymentProperties paymentProperties;

    @Autowired
    private SysConfigPaymentService configPaymentService;

    @Autowired
    private WxPayService wxPayService;

    @Override
    public BaseResult<Map<String, Object>> getOpenAuthMsg(OpenAuth openAuth) {
        return null;
    }

    @Override
    public BaseResult<Payment> executePayment(Payment payment) {
        String APP_ID = configPaymentService.getPaymentConfig(OpenEnum.ORIGINAL, WxpayConstant.WX_APPID).getParamValue();
        String MERCHANT_ID = configPaymentService.getPaymentConfig(OpenEnum.ORIGINAL, WxpayConstant.WX_MERCHANT_ID).getParamValue();

        WxPayUnifiedOrderV3Request request = new WxPayUnifiedOrderV3Request();
        request.setAppid(APP_ID);
        request.setMchid(MERCHANT_ID);
        request.setDescription(payment.getOrderDesc());
        request.setOutTradeNo(payment.getPaymentNo());
        request.setNotifyUrl(paymentProperties.getNotifyUrl()
                + "/" + payment.getOpenType().getCode()
                + "/" + payment.getPayChannel().getCode()
                + "/executePayment");

        if (StringUtil.isEmpty(payment.getPayAmt())) {
            return BaseResult.failed("微信APP支付金额异常");
        }

        BigDecimal payAmt = new BigDecimal(payment.getPayAmt()).multiply(new BigDecimal("100"));

        WxPayUnifiedOrderV3Request.Amount amount = new WxPayUnifiedOrderV3Request.Amount();
        amount.setTotal(payAmt.abs().intValue());
        request.setAmount(amount);

        try {
            Object orderV3 = wxPayService.createOrderV3(TradeTypeEnum.APP, request);
            String prepayid = ObjectUtil.getFieldValueByFieldName("prepayid", orderV3);
            payment.setPayInfo(StringUtil.get(prepayid));

            BaseResult<Payment> paymentBaseResult = queryPayment(payment);
            if (paymentBaseResult.getStatus()) {
                payment.setId(paymentBaseResult.getData().getId());
            }
            return BaseResult.succeed(payment);
        } catch (WxPayException e) {
            log.error("微信APP支付请求失败:{}", e.toString());
            return BaseResult.failed(e.getReturnMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return BaseResult.failed("微信APP支付请求失败");
        }
    }

    @Override
    public BaseResult<Payment> queryPayment(Payment payment) {
        String MERCHANT_ID = configPaymentService.getPaymentConfig(OpenEnum.ORIGINAL, WxpayConstant.WX_MERCHANT_ID).getParamValue();

        WxPayOrderQueryV3Request param = new WxPayOrderQueryV3Request();
        param.setMchid(MERCHANT_ID);
        if (!StringUtil.isEmpty(payment.getId())) {
            param.setTransactionId(payment.getId());
        }
        param.setOutTradeNo(payment.getPaymentNo());

        try {
            WxPayOrderQueryV3Result res = wxPayService.queryOrderV3(param);

            payment.setId(res.getTransactionId());
            String tradeState = res.getTradeState();
            if ("SUCCESS".equals(tradeState)) {
                payment.setStatus(TradeStatusEnum.succeeded);
            } else if ("REFUND".equals(tradeState)) {
                payment.setStatus(TradeStatusEnum.close);
            } else if ("NOTPAY".equals(tradeState)) {
                payment.setStatus(TradeStatusEnum.pending);
            } else if ("CLOSED".equals(tradeState)) {
                payment.setStatus(TradeStatusEnum.close);
            }
            return BaseResult.succeed(payment);
        } catch (WxPayException e) {
            log.error("微信APP支付查询请求失败:{}", e.toString());
            return BaseResult.failed(e.getReturnMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return BaseResult.failed("微信APP支付查询请求失败");
        }
    }

    @Override
    public BaseResult<ClosePayment> closePayment(ClosePayment closePayment) {
        String MERCHANT_ID = configPaymentService.getPaymentConfig(OpenEnum.ORIGINAL, WxpayConstant.WX_MERCHANT_ID).getParamValue();

        WxPayOrderCloseV3Request param = new WxPayOrderCloseV3Request();
        param.setMchid(MERCHANT_ID);
        param.setOutTradeNo(closePayment.getPaymentNo());

        try {
            wxPayService.closeOrderV3(param);
            return BaseResult.succeed(closePayment);
        } catch (WxPayException e) {
            log.error("微信APP支付关单请求失败:{}", e.toString());
            return BaseResult.failed(e.getReturnMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return BaseResult.failed("微信APP支付关单请求失败");
        }
    }

    @Override
    public BaseResult<RefundBill> refundPayment(RefundBill refundBill) {
        WxPayRefundV3Request param = new WxPayRefundV3Request();
        param.setTransactionId(refundBill.getTradeNo());
        param.setOutTradeNo(refundBill.getPaymentNo());
        param.setOutRefundNo(refundBill.getRefundNo());
        param.setReason(refundBill.getReason());

        param.setNotifyUrl(paymentProperties.getNotifyUrl()
                + "/" + refundBill.getOpenType().getCode()
                + "/" + refundBill.getPayChannel().getCode()
                + "/refundPayment");

        WxPayRefundV3Request.Amount amount = new WxPayRefundV3Request.Amount();
        BigDecimal refundAmt = refundBill.getRefundAmt().multiply(new BigDecimal("100"));
        amount.setRefund(refundAmt.intValue());

        BigDecimal orderAmt = refundBill.getOrderAmt().multiply(new BigDecimal("100"));
        amount.setTotal(orderAmt.intValue());

        amount.setCurrency("CNY");
        param.setAmount(amount);

        try {
            WxPayRefundV3Result res = wxPayService.refundV3(param);

            String refund_id = res.getRefundId();
            refundBill.setId(refund_id);
            return BaseResult.succeed(refundBill);
        } catch (WxPayException e) {
            log.error("微信APP支付请求失败:{}", e.toString());
            return BaseResult.failed(e.getReturnMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return BaseResult.failed("微信APP支付关单请求失败");
        }
    }

    @Override
    public BaseResult<PaymentCallBack> analysisNotice(OpenEnum openType, PayChannelEnum payChannel, MethodNameEnum methodName, HttpServletRequest request) {
        //支付回调
        SignatureHeader header = new SignatureHeader();
        header.setTimeStamp(request.getHeader("Wechatpay-Timestamp"));
        header.setNonce(request.getHeader("Wechatpay-Nonce"));
        header.setSerial(request.getHeader("Wechatpay-Serial"));
        header.setSignature(request.getHeader("Wechatpay-Signature"));

        String params = WebUtil.getBodyData(request);

        PaymentCallBack paymentCallBack = new PaymentCallBack();
        paymentCallBack.setMethodName(methodName);

        Map<String, Object> resMsg = new HashMap<>();
        try {
            if (MethodNameEnum.executePayment.equals(methodName)) {
                WxPayOrderNotifyV3Result wxPayOrderNotifyV3Result = wxPayService.parseOrderNotifyV3Result(params, header);
                //封装支付回调对象
                WxPayOrderNotifyV3Result.DecryptNotifyResult result = wxPayOrderNotifyV3Result.getResult();
                paymentCallBack.setId(result.getTransactionId());
                paymentCallBack.setPaymentNo(result.getOutTradeNo());

                String tradeState = result.getTradeState();
                if ("SUCCESS".equals(tradeState)) {
                    paymentCallBack.setStatus(TradeStatusEnum.succeeded);
                } else if ("NOTPAY".equals(tradeState) || "CLOSED".equals(tradeState) || "REFUND".equals(tradeState)) {
                    paymentCallBack.setStatus(TradeStatusEnum.close);
                    paymentCallBack.setErrMsg(result.getTradeStateDesc());
                } else if ("PAYERROR".equals(tradeState)) {
                    paymentCallBack.setStatus(TradeStatusEnum.failed);
                    paymentCallBack.setErrMsg(result.getTradeStateDesc());
                }

                ExecutePaymentCallBack executePayment = new ExecutePaymentCallBack();
                if(!StringUtil.isEmpty(result.getAmount().getTotal())){
                    executePayment.setPayAmt(new BigDecimal(result.getAmount().getTotal()).divide(new BigDecimal("100")).toString());
                }
                if(!StringUtil.isEmpty(result.getSuccessTime())){
                    Date date = new DateTime(result.getSuccessTime()).toDate();
                    executePayment.setTradeTime(date);
                }
                paymentCallBack.setExecutePaymentCallBack(executePayment);
            } else if (MethodNameEnum.refundPayment.equals(methodName)) {
                WxPayRefundNotifyV3Result wxPayRefundNotifyV3Result = wxPayService.parseRefundNotifyV3Result(params, header);
                //封装退款回调对象
                WxPayRefundNotifyV3Result.DecryptNotifyResult result = wxPayRefundNotifyV3Result.getResult();
                paymentCallBack.setId(result.getTransactionId());
                paymentCallBack.setPaymentNo(result.getOutTradeNo());

                String refundStatus = result.getRefundStatus();
                if ("SUCCESS".equals(refundStatus)) {
                    paymentCallBack.setStatus(TradeStatusEnum.succeeded);
                } else if ("CLOSED".equals(refundStatus)) {
                    paymentCallBack.setStatus(TradeStatusEnum.close);
                    paymentCallBack.setErrMsg("退款关闭");
                } else if ("ABNORMAL".equals(refundStatus)) {
                    paymentCallBack.setStatus(TradeStatusEnum.failed);
                    paymentCallBack.setErrMsg("退款失败");
                }

                RefundPaymentCallBack refundPayment = new RefundPaymentCallBack();
                refundPayment.setBillNo(result.getOutRefundNo());
                refundPayment.setTransNo(result.getRefundId());
                refundPayment.setRefundAmt(result.getAmount().getRefund());

                if(!StringUtil.isEmpty(result.getSuccessTime())){
                    Date date = new DateTime(result.getSuccessTime()).toDate();
                    refundPayment.setTradeTime(date);
                }
                paymentCallBack.setRefundPaymentCallBack(refundPayment);
            }
            resMsg.put("code", "SUCCESS");
            paymentCallBack.setResMsg(resMsg);
            return BaseResult.succeed(paymentCallBack);
        } catch (WxPayException e) {
            resMsg.put("code", "FAIL");
            resMsg.put("message", "失败");
            paymentCallBack.setResMsg(resMsg);
            log.error("微信回调对象转换失败, param is {} , err is {}", params, e.toString());
            return BaseResult.failed(HttpStatus.INTERNAL_SERVER_ERROR, paymentCallBack, "微信回调对象转换失败");
        } catch (Exception e) {
            e.printStackTrace();
            resMsg.put("code", "FAIL");
            resMsg.put("message", "失败");
            paymentCallBack.setResMsg(resMsg);
            return BaseResult.failed(HttpStatus.INTERNAL_SERVER_ERROR, paymentCallBack, "解析异常");
        }
    }
}
